part of '../bolter_provider.dart';

class PresenterProvider<P extends Presenter<P>> extends SingleChildStatefulWidget {
  final P presenter;
  final Widget child;

  const PresenterProvider({
    Key? key,
    required this.child,
    required this.presenter,
  }) : super(key: key);

  static P of<P extends Presenter<P>>(BuildContext context) => Provider.of<P>(context, listen: false);

  @override
  PresenterProviderState<P> createState() => PresenterProviderState<P>();
}

class PresenterProviderState<P extends Presenter<P>> extends SingleChildState<PresenterProvider<P>> {

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance!.endOfFrame.then(
      (_) {
        if (mounted) {
          widget.presenter.onLayout();
        }
      },
    );
    widget.presenter.context = context;
    widget.presenter._bolter = BolterProviderState._bolter;
    widget.presenter.initState();
  }

  @override
  Widget buildWithChild(BuildContext context, Widget? child) {
    return Provider<P>(
      lazy: false,
      create: (ctx) => widget.presenter,
      dispose: (_, presenter) => presenter.dispose(),
      child: widget.child,
    );
  }
}

class MultiPresenterProvider extends StatelessWidget {
  final List<SingleChildWidget> providers;
  final Widget child;

  const MultiPresenterProvider({
    Key? key,
    required this.providers,
    required this.child,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: providers,
      child: child,
    );
  }
}

extension PresenterProviderExtension on BuildContext {
  P presenter<P extends Presenter<P>>() => PresenterProvider.of<P>(this);
}
